"""RabbitMQ dependency keys and module."""

from .keys import RabbitMQClientDepKey
from .module import RabbitMQDepsModule

# ----------------------- #

__all__ = ["RabbitMQDepsModule", "RabbitMQClientDepKey"]
