"""Composition layer for assembling usecases, registries, and facades.

Provides document composition (:mod:`document`) and storage composition
(:mod:`storage`). Document composition includes facades, factories for plans
and registries, and operation identifiers. Storage composition provides
operation identifiers for object storage usecases.
"""
